Module: CollectiveIdea::Acts::NestedSet::Base::ClassMethods

Defined in:
lib/awesome_nested_set/base.rb

Instance Method Summary collapse

Instance Method Details

#after_move(*args, &block) ⇒ Object



234
235
236
# File 'lib/awesome_nested_set/base.rb', line 234

def after_move(*args, &block)
  set_callback :move, :after, *args, &block
end

#all_roots_valid?Boolean

Wrapper for each_root_valid? that can deal with scope.

Returns:



149
150
151
152
153
154
155
156
157
# File 'lib/awesome_nested_set/base.rb', line 149

def all_roots_valid?
  if acts_as_nested_set_options[:scope]
    roots(:group => scope_column_names).group_by{|record| scope_column_names.collect{|col| record.send(col.to_sym)}}.all? do |scope, grouped_roots|
      each_root_valid?(grouped_roots)
    end
  else
    each_root_valid?(roots)
  end
end

#before_move(*args, &block) ⇒ Object



230
231
232
# File 'lib/awesome_nested_set/base.rb', line 230

def before_move(*args, &block)
  set_callback :move, :before, *args, &block
end

#each_root_valid?(roots_to_validate) ⇒ Boolean

Returns:



159
160
161
162
163
164
165
166
167
# File 'lib/awesome_nested_set/base.rb', line 159

def each_root_valid?(roots_to_validate)
  left = right = 0
  roots_to_validate.all? do |root|
    (root.left > left && root.right > right).tap do
      left = root.left
      right = root.right
    end
  end
end

#each_with_level(objects) ⇒ Object

Iterates over tree elements and determines the current level in the tree. Only accepts default ordering, odering by an other column than lft does not work. This method is much more efficent than calling level because it doesn't require any additional database queries.

Example:

Category.each_with_level(Category.root.self_and_descendants) do |o, level|


214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
# File 'lib/awesome_nested_set/base.rb', line 214

def each_with_level(objects)
  path = [nil]
  objects.each do |o|
    if o.parent_id != path.last
      # we are on a new level, did we decent or ascent?
      if path.include?(o.parent_id)
        # remove wrong wrong tailing paths elements
        path.pop while path.last != o.parent_id
      else
        path << o.parent_id
      end
    end
    yield(o, path.length - 1)
  end
end

#left_and_rights_valid?Boolean

Returns:



120
121
122
123
124
125
126
127
128
129
130
131
132
133
# File 'lib/awesome_nested_set/base.rb', line 120

def left_and_rights_valid?
  count(
    :joins => "LEFT OUTER JOIN #{quoted_table_name} AS parent ON " +
      "#{quoted_table_name}.#{quoted_parent_column_name} = parent.#{primary_key}",
    :conditions =>
      "#{quoted_table_name}.#{quoted_left_column_name} IS NULL OR " +
      "#{quoted_table_name}.#{quoted_right_column_name} IS NULL OR " +
      "#{quoted_table_name}.#{quoted_left_column_name} >= " +
        "#{quoted_table_name}.#{quoted_right_column_name} OR " +
      "(#{quoted_table_name}.#{quoted_parent_column_name} IS NOT NULL AND " +
        "(#{quoted_table_name}.#{quoted_left_column_name} <= parent.#{quoted_left_column_name} OR " +
        "#{quoted_table_name}.#{quoted_right_column_name} >= parent.#{quoted_right_column_name}))"
  ) == 0
end

#no_duplicates_for_columns?Boolean

Returns:



135
136
137
138
139
140
141
142
143
144
145
146
# File 'lib/awesome_nested_set/base.rb', line 135

def no_duplicates_for_columns?
  scope_string = Array(acts_as_nested_set_options[:scope]).map do |c|
    connection.quote_column_name(c)
  end.push(nil).join(", ")
  [quoted_left_column_name, quoted_right_column_name].all? do |column|
    # No duplicates
    first(
      :select => "#{scope_string}#{column}, COUNT(#{column})",
      :group => "#{scope_string}#{column}
        HAVING COUNT(#{column}) > 1").nil?
  end
end

#rebuild!Object

Rebuilds the left & rights if unset or invalid. Also very useful for converting from acts_as_tree.



170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
# File 'lib/awesome_nested_set/base.rb', line 170

def rebuild!
  # Don't rebuild a valid tree.
  return true if valid?

  scope = lambda{|node|}
  if acts_as_nested_set_options[:scope]
    scope = lambda{|node|
      scope_column_names.inject(""){|str, column_name|
        str << "AND #{connection.quote_column_name(column_name)} = #{connection.quote(node.send(column_name.to_sym))} "
      }
    }
  end
  indices = {}

  set_left_and_rights = lambda do |node|
    # set left
    node[left_column_name] = indices[scope.call(node)] += 1
    # find
    where("#{quoted_parent_column_name} = ? #{scope.call(node)}", node).
      order("#{quoted_left_column_name}, #{quoted_right_column_name}, id").
      all.each{|n| set_left_and_rights.call(n) }
    # set right
    node[right_column_name] = indices[scope.call(node)] += 1
    node.save!
  end

  # Find root node(s)
  root_nodes = where(parent_column_name => nil).
      order("#{quoted_left_column_name}, #{quoted_right_column_name}, id").
      all.each do |root_node|
    # setup index for this scope
    indices[scope.call(root_node)] ||= 0
    set_left_and_rights.call(root_node)
  end
end

#rootObject

Returns the first root



112
113
114
# File 'lib/awesome_nested_set/base.rb', line 112

def root
  roots.first
end

#valid?Boolean

Returns:



116
117
118
# File 'lib/awesome_nested_set/base.rb', line 116

def valid?
  left_and_rights_valid? && no_duplicates_for_columns? && all_roots_valid?
end