Module: CollectiveIdea::Acts::NestedSet::Base::InstanceMethods

Defined in:
lib/awesome_nested_set/base.rb

Overview

Any instance method that returns a collection makes use of Rails 2.1's named_scope (which is bundled for Rails 2.0), so it can be treated as a finder.

category.self_and_descendants.count
category.ancestors.find(:all, :conditions => "name like '%foo%'")

Instance Method Summary collapse

Instance Method Details

#<=>(x) ⇒ Object

order by left column



309
310
311
# File 'lib/awesome_nested_set/base.rb', line 309

def <=>(x)
  left <=> x.left
end

#==(comparison_object) ⇒ Object

Redefine to act like active record



314
315
316
317
318
319
# File 'lib/awesome_nested_set/base.rb', line 314

def ==(comparison_object)
  comparison_object.equal?(self) ||
    (comparison_object.instance_of?(self.class) &&
      comparison_object.id == id &&
      !comparison_object.new_record?)
end

#ancestorsObject

Returns an array of all parents



334
335
336
# File 'lib/awesome_nested_set/base.rb', line 334

def ancestors
  without_self self_and_ancestors
end

#child?Boolean

Returns true is this is a child node

Returns:

  • (Boolean)


304
305
306
# File 'lib/awesome_nested_set/base.rb', line 304

def child?
  !parent_id.nil?
end

#descendantsObject

Returns a set of all of its children and nested children



367
368
369
# File 'lib/awesome_nested_set/base.rb', line 367

def descendants
  without_self self_and_descendants
end

#is_ancestor_of?(other) ⇒ Boolean

Returns:

  • (Boolean)


379
380
381
# File 'lib/awesome_nested_set/base.rb', line 379

def is_ancestor_of?(other)
  self.left < other.left && other.left < self.right && same_scope?(other)
end

#is_descendant_of?(other) ⇒ Boolean

Returns:

  • (Boolean)


371
372
373
# File 'lib/awesome_nested_set/base.rb', line 371

def is_descendant_of?(other)
  other.left < self.left && self.left < other.right && same_scope?(other)
end

#is_or_is_ancestor_of?(other) ⇒ Boolean

Returns:

  • (Boolean)


383
384
385
# File 'lib/awesome_nested_set/base.rb', line 383

def is_or_is_ancestor_of?(other)
  self.left <= other.left && other.left < self.right && same_scope?(other)
end

#is_or_is_descendant_of?(other) ⇒ Boolean

Returns:

  • (Boolean)


375
376
377
# File 'lib/awesome_nested_set/base.rb', line 375

def is_or_is_descendant_of?(other)
  other.left <= self.left && self.left < other.right && same_scope?(other)
end

#leaf?Boolean

Returns:

  • (Boolean)


299
300
301
# File 'lib/awesome_nested_set/base.rb', line 299

def leaf?
  !new_record? && right - left == 1
end

#leavesObject

Returns a set of all of its nested children which do not have children



349
350
351
# File 'lib/awesome_nested_set/base.rb', line 349

def leaves
  descendants.scoped.where("#{self.class.quoted_table_name}.#{quoted_right_column_name} - #{self.class.quoted_table_name}.#{quoted_left_column_name} = 1")
end

#leftObject

Value of the left column



285
286
287
# File 'lib/awesome_nested_set/base.rb', line 285

def left
  self[left_column_name]
end

#left_siblingObject

Find the first sibling to the left



395
396
397
398
# File 'lib/awesome_nested_set/base.rb', line 395

def left_sibling
  siblings.first(:conditions => ["#{self.class.quoted_table_name}.#{quoted_left_column_name} < ?", left],
    :order => "#{self.class.quoted_table_name}.#{quoted_left_column_name} DESC")
end

#levelObject

Returns the level of this object in the tree root level is 0



355
356
357
# File 'lib/awesome_nested_set/base.rb', line 355

def level
  parent_id.nil? ? 0 : ancestors.count
end

#move_leftObject

Shorthand method for finding the left sibling and moving to the left of it.



406
407
408
# File 'lib/awesome_nested_set/base.rb', line 406

def move_left
  move_to_left_of left_sibling
end

#move_possible?(target) ⇒ Boolean

Returns:

  • (Boolean)


435
436
437
438
439
440
441
# File 'lib/awesome_nested_set/base.rb', line 435

def move_possible?(target)
  self != target && # Can't target self
  same_scope?(target) && # can't be in different scopes
  # !(left..right).include?(target.left..target.right) # this needs tested more
  # detect impossible move
  !((left <= target.left && right >= target.left) or (left <= target.right && right >= target.right))
end

#move_rightObject

Shorthand method for finding the right sibling and moving to the right of it.



411
412
413
# File 'lib/awesome_nested_set/base.rb', line 411

def move_right
  move_to_right_of right_sibling
end

#move_to_child_of(node) ⇒ Object

Move the node to the child of another node (you can pass id only)



426
427
428
# File 'lib/awesome_nested_set/base.rb', line 426

def move_to_child_of(node)
  move_to node, :child
end

#move_to_left_of(node) ⇒ Object

Move the node to the left of another node (you can pass id only)



416
417
418
# File 'lib/awesome_nested_set/base.rb', line 416

def move_to_left_of(node)
  move_to node, :left
end

#move_to_right_of(node) ⇒ Object

Move the node to the left of another node (you can pass id only)



421
422
423
# File 'lib/awesome_nested_set/base.rb', line 421

def move_to_right_of(node)
  move_to node, :right
end

#move_to_rootObject

Move the node to root nodes



431
432
433
# File 'lib/awesome_nested_set/base.rb', line 431

def move_to_root
  move_to nil, :root
end

#parent_idObject

Value of the parent column



280
281
282
# File 'lib/awesome_nested_set/base.rb', line 280

def parent_id
  self[parent_column_name]
end

#rightObject

Value of the right column



290
291
292
# File 'lib/awesome_nested_set/base.rb', line 290

def right
  self[right_column_name]
end

#right_siblingObject

Find the first sibling to the right



401
402
403
# File 'lib/awesome_nested_set/base.rb', line 401

def right_sibling
  siblings.first(:conditions => ["#{self.class.quoted_table_name}.#{quoted_left_column_name} > ?", left])
end

#rootObject

Returns root



322
323
324
# File 'lib/awesome_nested_set/base.rb', line 322

def root
  self_and_ancestors.first
end

#root?Boolean

Returns true if this is a root node.

Returns:

  • (Boolean)


295
296
297
# File 'lib/awesome_nested_set/base.rb', line 295

def root?
  parent_id.nil?
end

#same_scope?(other) ⇒ Boolean

Check if other model is in the same scope

Returns:

  • (Boolean)


388
389
390
391
392
# File 'lib/awesome_nested_set/base.rb', line 388

def same_scope?(other)
  Array(acts_as_nested_set_options[:scope]).all? do |attr|
    self.send(attr) == other.send(attr)
  end
end

#self_and_ancestorsObject

Returns the array of all parents and self



327
328
329
330
331
# File 'lib/awesome_nested_set/base.rb', line 327

def self_and_ancestors
  nested_set_scope.scoped.where(
    "#{self.class.quoted_table_name}.#{quoted_left_column_name} <= ? AND #{self.class.quoted_table_name}.#{quoted_right_column_name} >= ?", left, right
  )
end

#self_and_descendantsObject

Returns a set of itself and all of its nested children



360
361
362
363
364
# File 'lib/awesome_nested_set/base.rb', line 360

def self_and_descendants
  nested_set_scope.scoped.where(
    "#{self.class.quoted_table_name}.#{quoted_left_column_name} >= ? AND #{self.class.quoted_table_name}.#{quoted_right_column_name} <= ?", left, right
  )
end

#self_and_siblingsObject

Returns the array of all children of the parent, including self



339
340
341
# File 'lib/awesome_nested_set/base.rb', line 339

def self_and_siblings
  nested_set_scope.scoped.where(parent_column_name => parent_id)
end

#siblingsObject

Returns the array of all children of the parent, except self



344
345
346
# File 'lib/awesome_nested_set/base.rb', line 344

def siblings
  without_self self_and_siblings
end

#to_textObject



443
444
445
446
447
# File 'lib/awesome_nested_set/base.rb', line 443

def to_text
  self_and_descendants.map do |node|
    "#{'*'*(node.level+1)} #{node.id} #{node.to_s} (#{node.parent_id}, #{node.left}, #{node.right})"
  end.join("\n")
end